@@ -94,7 +94,7 @@ gst_ffmpeg_set_palette (GstCaps *caps, AVCodecContext *context)
     gst_caps_new_simple (mimetype,			      	\
 	"width",     GST_TYPE_INT_RANGE, 16, 4096,      	\
 	"height",    GST_TYPE_INT_RANGE, 16, 4096,	      	\
-	"framerate", GST_TYPE_DOUBLE_RANGE, (double) 0., 	\
+	"framerate", GST_TYPE_DOUBLE_RANGE, (double) 1., 	\
 						G_MAXDOUBLE,	\
 	__VA_ARGS__, NULL)
 
@@ -462,6 +462,17 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
       }
       break;
 
+    case CODEC_ID_QTRLE:
+      caps = GST_FF_VID_CAPS_NEW ("video/x-rle",
+          "layout", G_TYPE_STRING, "quicktime", NULL);
+      if (context) {
+        gst_caps_set_simple (caps,
+            "depth", G_TYPE_INT, (gint) context->bits_per_sample, NULL);
+      } else {
+        gst_caps_set_simple (caps, "depth", GST_TYPE_INT_RANGE, 1, 64, NULL);
+      }
+      break;
+
     case CODEC_ID_MSVIDEO1:
       caps = GST_FF_VID_CAPS_NEW ("video/x-msvideocodec",
 	  "msvideoversion", G_TYPE_INT, 1, NULL);
@@ -473,17 +484,21 @@ gst_ffmpeg_codecid_to_caps (enum CodecID codec_id,
           "wmvversion", G_TYPE_INT, 3, NULL);
       break;
 
+    case CODEC_ID_QDM2:
+      caps = GST_FF_AUD_CAPS_NEW ("audio/x-qdm2", NULL);
+      break;
+
     case CODEC_ID_WS_VQA:
     case CODEC_ID_IDCIN:
     case CODEC_ID_8BPS:
     case CODEC_ID_SMC:
     case CODEC_ID_FLIC:
     case CODEC_ID_TRUEMOTION1:
+    case CODEC_ID_TRUEMOTION2:
     case CODEC_ID_VMDVIDEO:
     case CODEC_ID_VMDAUDIO:
     case CODEC_ID_MSZH:
     case CODEC_ID_ZLIB:
-    case CODEC_ID_QTRLE:
     case CODEC_ID_SONIC:
     case CODEC_ID_SONIC_LS:
     case CODEC_ID_SNOW:
@@ -1562,6 +1577,9 @@ gst_ffmpeg_caps_to_codecid (const GstCaps * caps, AVCodecContext * context)
   } else if (!strcmp (mimetype, "audio/x-vorbis")) {
     id = CODEC_ID_VORBIS;
     audio = TRUE;
+  } else if (!strcmp (mimetype, "audio/x-qdm2")) {
+    id = CODEC_ID_QDM2;
+    audio = TRUE;
   } else if (!strcmp (mimetype, "audio/mpeg")) {
     gint layer = 0;
     gint mpegversion = 0;
@@ -1923,6 +1941,9 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_VORBIS:
       name = "Vorbis audio";
       break;
+    case CODEC_ID_QDM2:
+      name = "QDesign Music 2";
+      break;
     case CODEC_ID_AC3:
       name = "AC-3 audio";
       break;
@@ -2083,6 +2104,9 @@ gst_ffmpeg_get_codecid_longname (enum CodecID codec_id)
     case CODEC_ID_TRUEMOTION1:
       name = "Duck Truemotion video";
       break;
+    case CODEC_ID_TRUEMOTION2:
+      name = "Duck Truemotion 2 video";
+      break;
     case CODEC_ID_VMDVIDEO:
       name = "Sierra VMD video";
       break;
