@@ -53,7 +53,8 @@ struct _GstFFMpegDec
   gboolean opened;
   union {
     struct {
-      gint width, height, fps, fps_base;
+      gint width, height;
+      gdouble fps, old_fps;
       enum PixelFormat pix_fmt;
     } video;
     struct {
@@ -72,6 +73,7 @@ struct _GstFFMpegDec
   GValue *par;		/* pixel aspect ratio of incoming data */
 
   gint hurry_up, lowres;
+  GMutex *mainlock;
 };
 
 typedef struct _GstFFMpegDecClass GstFFMpegDecClass;
@@ -116,7 +118,7 @@ static GHashTable *global_plugins;
 static void gst_ffmpegdec_base_init (GstFFMpegDecClass * klass);
 static void gst_ffmpegdec_class_init (GstFFMpegDecClass * klass);
 static void gst_ffmpegdec_init (GstFFMpegDec * ffmpegdec);
-static void gst_ffmpegdec_dispose (GObject * object);
+static void gst_ffmpegdec_finalize (GObject * object);
 
 static gboolean gst_ffmpegdec_query (GstPad * pad, GstQueryType type,
     GstFormat * fmt, gint64 * value);
@@ -246,7 +248,7 @@ gst_ffmpegdec_class_init (GstFFMpegDecClass * klass)
           "At which resolution to decode images",
           GST_FFMPEGDEC_TYPE_LOWRES, 0, G_PARAM_READWRITE));
 
-  gobject_class->dispose = gst_ffmpegdec_dispose;
+  gobject_class->finalize = gst_ffmpegdec_finalize;
   gobject_class->set_property = gst_ffmpegdec_set_property;
   gobject_class->get_property = gst_ffmpegdec_get_property;
   gstelement_class->change_state = gst_ffmpegdec_change_state;
@@ -283,22 +285,28 @@ gst_ffmpegdec_init (GstFFMpegDec * ffmpegdec)
   ffmpegdec->hurry_up = ffmpegdec->lowres = 0;
 
   ffmpegdec->last_buffer = NULL;
+  ffmpegdec->mainlock = g_mutex_new ();
+
+  ffmpegdec->format.video.fps = -1;
+  ffmpegdec->format.video.old_fps = -1;
 
   GST_FLAG_SET (ffmpegdec, GST_ELEMENT_EVENT_AWARE);
 }
 
 static void
-gst_ffmpegdec_dispose (GObject * object)
+gst_ffmpegdec_finalize (GObject * object)
 {
   GstFFMpegDec *ffmpegdec = (GstFFMpegDec *) object;
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-  /* old session should have been closed in element_class->dispose */
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+  /* old session should have been closed in element_class->finalize */
   g_assert (!ffmpegdec->opened);
 
   /* clean up remaining allocated data */
   av_free (ffmpegdec->context);
   av_free (ffmpegdec->picture);
+  
+  g_mutex_free (ffmpegdec->mainlock);
 }
 
 static gboolean
@@ -381,6 +389,9 @@ gst_ffmpegdec_close (GstFFMpegDec *ffmpegdec)
     av_parser_close (ffmpegdec->pctx);
     ffmpegdec->pctx = NULL;
   }
+
+  ffmpegdec->format.video.fps = -1;
+  ffmpegdec->format.video.old_fps = -1;
 }
 
 static gboolean
@@ -413,8 +424,6 @@ gst_ffmpegdec_open (GstFFMpegDec *ffmpegdec)
     case CODEC_TYPE_VIDEO:
       ffmpegdec->format.video.width = 0;
       ffmpegdec->format.video.height = 0;
-      ffmpegdec->format.video.fps = 0;
-      ffmpegdec->format.video.fps_base = 0;
       ffmpegdec->format.video.pix_fmt = PIX_FMT_NB;
       break;
     case CODEC_TYPE_AUDIO:
@@ -440,6 +449,8 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   const GValue *par;
 
+  g_mutex_lock (ffmpegdec->mainlock);
+
   /* close old session */
   gst_ffmpegdec_close (ffmpegdec);
 
@@ -453,7 +464,8 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
   /* get size and so */
   gst_ffmpeg_caps_with_codecid (oclass->in_plugin->id,
       oclass->in_plugin->type, caps, ffmpegdec->context);
-  if (!ffmpegdec->context->time_base.den) {
+  if (!ffmpegdec->context->time_base.den ||
+      !ffmpegdec->context->time_base.num) {
     ffmpegdec->context->time_base.num = 1;
     ffmpegdec->context->time_base.den = 25;
   }
@@ -466,6 +478,13 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
     ffmpegdec->par = g_new0 (GValue, 1);
     gst_value_init_and_copy (ffmpegdec->par, par);
   }
+  if (gst_structure_has_field (structure, "framerate")) {
+    ffmpegdec->format.video.old_fps = ffmpegdec->format.video.fps;
+    gst_structure_get_double (structure, "framerate", &ffmpegdec->format.video.fps);
+  }else {
+    ffmpegdec->format.video.old_fps = ffmpegdec->format.video.fps;
+    ffmpegdec->format.video.fps = -1;
+  }
 
   /* do *not* draw edges */
   ffmpegdec->context->flags |= CODEC_FLAG_EMU_EDGE;
@@ -485,9 +504,12 @@ gst_ffmpegdec_connect (GstPad * pad, const GstCaps * caps)
       g_free (ffmpegdec->par);
       ffmpegdec->par = NULL;
     }
+
+    g_mutex_unlock (ffmpegdec->mainlock);
     return GST_PAD_LINK_REFUSED;
   }
 
+  g_mutex_unlock (ffmpegdec->mainlock);
   return GST_PAD_LINK_OK;
 }
 
@@ -588,22 +610,18 @@ gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec)
     case CODEC_TYPE_VIDEO:
       if (ffmpegdec->format.video.width == ffmpegdec->context->width &&
           ffmpegdec->format.video.height == ffmpegdec->context->height &&
-          ffmpegdec->format.video.fps == ffmpegdec->context->time_base.den &&
-          ffmpegdec->format.video.fps_base ==
-              ffmpegdec->context->time_base.num &&
+          ffmpegdec->format.video.fps == ffmpegdec->format.video.old_fps &&
 	  ffmpegdec->format.video.pix_fmt == ffmpegdec->context->pix_fmt)
         return TRUE;
-      GST_DEBUG ("Renegotiating video from %dx%d@%d/%dfps to %dx%d@%d/%dfps",
-          ffmpegdec->format.video.width, ffmpegdec->format.video.height,
-          ffmpegdec->format.video.fps, ffmpegdec->format.video.fps_base,
-          ffmpegdec->context->width, ffmpegdec->context->height,
-          ffmpegdec->context->time_base.den,
-          ffmpegdec->context->time_base.num);
+      GST_DEBUG ("Renegotiating video from %dx%d@%0.2ffps to %dx%d@%0.2ffps",
+		 ffmpegdec->format.video.width, ffmpegdec->format.video.height,
+		 ffmpegdec->format.video.old_fps,
+		 ffmpegdec->context->width, ffmpegdec->context->height,
+		 ffmpegdec->format.video.fps);
       ffmpegdec->format.video.width = ffmpegdec->context->width;
       ffmpegdec->format.video.height = ffmpegdec->context->height;
-      ffmpegdec->format.video.fps = ffmpegdec->context->time_base.den;
-      ffmpegdec->format.video.fps_base = ffmpegdec->context->time_base.num;
-      ffmpegdec->format.video.pix_fmt = ffmpegdec->context->pix_fmt;
+      ffmpegdec->format.video.old_fps = ffmpegdec->format.video.fps;
+      ffmpegdec->format.video.pix_fmt = ffmpegdec->context->pix_fmt;      
       break;
     case CODEC_TYPE_AUDIO:
       if (ffmpegdec->format.audio.samplerate ==
@@ -627,7 +645,17 @@ gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec)
    * prefer ffmpeg par over sink par (since it's provided
    * by the codec, which is more often correct).
    */
+
  if (caps) {
+   
+   /* if a demuxer provided a framerate then use it (#313970) */
+   if (ffmpegdec->format.video.fps != -1) {
+     gst_structure_set (gst_caps_get_structure (caps, 0),
+			"framerate", G_TYPE_DOUBLE, 
+			ffmpegdec->format.video.fps, 
+			NULL);     
+   }
+   
    if (ffmpegdec->context->sample_aspect_ratio.num &&
        ffmpegdec->context->sample_aspect_ratio.den) {
      GST_DEBUG ("setting ffmpeg provided pixel-aspect-ratio");
@@ -665,7 +693,7 @@ gst_ffmpegdec_negotiate (GstFFMpegDec * ffmpegdec)
 
 static gint
 gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
-    guint8 * data, guint size, gint * got_data, guint64 * in_ts)
+    guint8 * data, guint size, gint * got_data, guint64 * in_ts, GstBuffer *inbuf)
 {
   GstFFMpegDecClass *oclass =
       (GstFFMpegDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
@@ -695,8 +723,6 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
 
 	if (ffmpegdec->picture->opaque != NULL) {
 	  outbuf = (GstBuffer *) ffmpegdec->picture->opaque;
-          if (outbuf == ffmpegdec->last_buffer)
-            ffmpegdec->last_buffer = NULL;
 	} else {
 	  AVPicture pic;
 	  gint fsize = gst_ffmpeg_avpicture_get_size (ffmpegdec->context->pix_fmt,
@@ -742,22 +768,29 @@ gst_ffmpegdec_frame (GstFFMpegDec * ffmpegdec,
 	} else {
 	  GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_DELTA_UNIT);
 	}
-		
-        GST_BUFFER_TIMESTAMP (outbuf) = ffmpegdec->next_ts;
-        if (ffmpegdec->context->time_base.num != 0 &&
-            ffmpegdec->context->time_base.den != 0) {
-          GST_BUFFER_DURATION (outbuf) = GST_SECOND *
+	
+	/* If we have used framerate from the demuxer than
+	 use also its timestamp informations (#317596) */
+	if ((ffmpegdec->format.video.fps != -1)&&
+	    (inbuf != NULL)) {
+	  gst_buffer_stamp (outbuf, inbuf);	  
+	}else {
+	  GST_BUFFER_TIMESTAMP (outbuf) = ffmpegdec->next_ts;
+	  if (ffmpegdec->context->time_base.num != 0 &&
+	      ffmpegdec->context->time_base.den != 0) {
+	    GST_BUFFER_DURATION (outbuf) = GST_SECOND *
               ffmpegdec->context->time_base.num /
               ffmpegdec->context->time_base.den;
-
-          /* Take repeat_pict into account */
-          GST_BUFFER_DURATION (outbuf) += GST_BUFFER_DURATION (outbuf)
+	         	  
+	    /* Take repeat_pict into account */
+	    GST_BUFFER_DURATION (outbuf) += GST_BUFFER_DURATION (outbuf)
               * ffmpegdec->picture->repeat_pict / 2;
-	  
-          ffmpegdec->next_ts += GST_BUFFER_DURATION (outbuf);
-        } else {
-          ffmpegdec->next_ts = GST_CLOCK_TIME_NONE;
-        }
+	    
+	    ffmpegdec->next_ts += GST_BUFFER_DURATION (outbuf);
+	  } else {
+	    ffmpegdec->next_ts = GST_CLOCK_TIME_NONE;
+	  }
+	}
       } else if (ffmpegdec->picture->pict_type != -1 &&
 		 oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
         /* update time for skip-frame */
@@ -868,7 +901,7 @@ gst_ffmpegdec_handle_event (GstFFMpegDec * ffmpegdec, GstEvent * event)
         gint have_data, len, try = 0;
         do {
           len = gst_ffmpegdec_frame (ffmpegdec, NULL, 0, &have_data,
-              &ffmpegdec->next_ts);
+              &ffmpegdec->next_ts, NULL);
           if (len < 0 || have_data == 0)
             break;
         } while (try++ < 10);
@@ -931,9 +964,12 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
   gint bsize, size, len, have_data;
   guint64 in_ts;
 
+  g_mutex_lock (ffmpegdec->mainlock);
+
   /* event handling */
   if (GST_IS_EVENT (_data)) {
     gst_ffmpegdec_handle_event (ffmpegdec, GST_EVENT (_data));
+    g_mutex_unlock (ffmpegdec->mainlock);
     return;
   }
 
@@ -944,6 +980,7 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
     GST_ELEMENT_ERROR (ffmpegdec, CORE, NEGOTIATION, (NULL),
         ("ffdec_%s: input format was not set before data start",
             oclass->in_plugin->name));
+    g_mutex_unlock (ffmpegdec->mainlock);
     return;
   }
 
@@ -1000,7 +1037,7 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
     }
 
     if ((len = gst_ffmpegdec_frame (ffmpegdec, data, size,
-             &have_data, &in_ts)) < 0)
+             &have_data, &in_ts, inbuf)) < 0)
       break;
 
     if (!ffmpegdec->pctx) {
@@ -1022,6 +1059,8 @@ gst_ffmpegdec_chain (GstPad * pad, GstData * _data)
   } else if (bsize > 0) {
     GST_DEBUG_OBJECT (ffmpegdec, "Dropping %d bytes of data", bsize);
   }
+
+  g_mutex_unlock (ffmpegdec->mainlock);
   gst_buffer_unref (inbuf);
 }
 
@@ -1117,7 +1156,7 @@ gst_ffmpegdec_register (GstPlugin * plugin)
     /* no quasi-codecs, please */
     if (in_plugin->id == CODEC_ID_RAWVIDEO ||
         (in_plugin->id >= CODEC_ID_PCM_S16LE &&
-            in_plugin->id <= CODEC_ID_PCM_U8)) {
+            in_plugin->id <= CODEC_ID_PCM_S24DAUD)) {
       goto next;
     }
 
