@@ -296,7 +296,7 @@ gst_ffmpegdemux_src_event (GstPad * pad, GstEvent * event)
       offset = GST_EVENT_SEEK_OFFSET (event);
       switch (GST_EVENT_SEEK_FORMAT (event)) {
         case GST_FORMAT_DEFAULT:
-          if (stream->codec.codec_type != CODEC_TYPE_VIDEO) {
+          if (stream->codec->codec_type != CODEC_TYPE_VIDEO) {
             res = FALSE;
             break;
           } else {
@@ -336,7 +336,7 @@ gst_ffmpegdemux_src_format_list (GstPad * pad)
   }, src_a_formats[] = {
   GST_FORMAT_TIME, 0};
 
-  return (stream->codec.codec_type == CODEC_TYPE_VIDEO) ?
+  return (stream->codec->codec_type == CODEC_TYPE_VIDEO) ?
       src_v_formats : src_a_formats;
 }
 
@@ -371,7 +371,7 @@ gst_ffmpegdemux_src_query (GstPad * pad,
           break;
         case GST_FORMAT_DEFAULT:
           if (stream->codec_info_nb_frames &&
-              stream->codec.codec_type == CODEC_TYPE_VIDEO) {
+              stream->codec->codec_type == CODEC_TYPE_VIDEO) {
             *value = stream->codec_info_nb_frames;
             res = TRUE;
           }
@@ -397,7 +397,7 @@ gst_ffmpegdemux_src_query (GstPad * pad,
           }
           break;
         case GST_FORMAT_DEFAULT:
-          if (stream && stream->codec.codec_type == CODEC_TYPE_VIDEO &&
+          if (stream && stream->codec->codec_type == CODEC_TYPE_VIDEO &&
               GST_CLOCK_TIME_IS_VALID (demux->last_ts[stream->index])) {
             res = gst_pad_convert (pad, GST_FORMAT_TIME,
                 demux->last_ts[stream->index], fmt, value);
@@ -429,15 +429,15 @@ gst_ffmpegdemux_src_convert (GstPad * pad,
   AVStream *stream = gst_ffmpegdemux_stream_from_pad (pad);
   gboolean res = TRUE;
 
-  if (!stream || stream->codec.codec_type != CODEC_TYPE_VIDEO)
+  if (!stream || stream->codec->codec_type != CODEC_TYPE_VIDEO)
     return FALSE;
 
   switch (src_fmt) {
     case GST_FORMAT_TIME:
       switch (*dest_fmt) {
         case GST_FORMAT_DEFAULT:
-          *dest_value = src_value * stream->r_frame_rate /
-              (GST_SECOND * stream->r_frame_rate_base);
+          *dest_value = src_value * stream->r_frame_rate.num /
+              (GST_SECOND * stream->r_frame_rate.den);
           break;
         default:
           res = FALSE;
@@ -447,8 +447,8 @@ gst_ffmpegdemux_src_convert (GstPad * pad,
     case GST_FORMAT_DEFAULT:
       switch (*dest_fmt) {
         case GST_FORMAT_TIME:
-          *dest_value = src_value * GST_SECOND * stream->r_frame_rate_base /
-              stream->r_frame_rate;
+          *dest_value = src_value * GST_SECOND * stream->r_frame_rate.num /
+              stream->r_frame_rate.den;
           break;
         default:
           res = FALSE;
@@ -473,8 +473,9 @@ gst_ffmpegdemux_add (GstFFMpegDemux * demux, AVStream * stream)
   GstCaps *caps;
   gint num;
   gchar *padname;
+  const gchar *codec;
 
-  switch (stream->codec.codec_type) {
+  switch (stream->codec->codec_type) {
     case CODEC_TYPE_VIDEO:
       templ = oclass->videosrctempl;
       num = demux->videopads++;
@@ -484,7 +485,7 @@ gst_ffmpegdemux_add (GstFFMpegDemux * demux, AVStream * stream)
       num = demux->audiopads++;
       break;
     default:
-      GST_WARNING ("Unknown pad type %d", stream->codec.codec_type);
+      GST_WARNING ("Unknown pad type %d", stream->codec->codec_type);
       break;
   }
   if (!templ)
@@ -508,11 +509,21 @@ gst_ffmpegdemux_add (GstFFMpegDemux * demux, AVStream * stream)
 
   /* get caps that belongs to this stream */
   caps =
-      gst_ffmpeg_codecid_to_caps (stream->codec.codec_id, &stream->codec, TRUE);
+      gst_ffmpeg_codecid_to_caps (stream->codec->codec_id, stream->codec, TRUE);
   gst_pad_set_explicit_caps (pad, caps);
 
   gst_element_add_pad (GST_ELEMENT (demux), pad);
 
+  /* metadata */
+  if ((codec = gst_ffmpeg_get_codecid_longname (stream->codec->codec_id))) {
+    GstTagList *list = gst_tag_list_new ();
+
+    gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+        (stream->codec->codec_type == CODEC_TYPE_VIDEO) ?
+        GST_TAG_VIDEO_CODEC : GST_TAG_AUDIO_CODEC, codec, NULL);
+    gst_element_found_tags_for_pad (GST_ELEMENT (demux), pad, 0, list);
+  }
+
   return TRUE;
 }
 
@@ -644,15 +655,16 @@ gst_ffmpegdemux_loop (GstElement * element)
     memcpy (GST_BUFFER_DATA (outbuf), pkt.data, pkt.size);
     GST_BUFFER_SIZE (outbuf) = pkt.size;
 
-    if (pkt.pts != AV_NOPTS_VALUE) {
-      GST_BUFFER_TIMESTAMP (outbuf) = (gdouble) (pkt.pts +
-          stream->start_time) * GST_SECOND / AV_TIME_BASE;
+    GST_BUFFER_TIMESTAMP (outbuf) =
+        gst_ffmpeg_time_ff_to_gst (pkt.pts,
+          demux->context->streams[pkt.stream_index]->time_base);
+    if (GST_BUFFER_TIMESTAMP_IS_VALID (outbuf))
       demux->last_ts[stream->index] = GST_BUFFER_TIMESTAMP (outbuf);
-    }
 
-    if (pkt.flags & PKT_FLAG_KEY) {
+    if (pkt.flags & PKT_FLAG_KEY)
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
-    }
+    else
+      GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_DELTA_UNIT);
 
     gst_pad_push (pad, GST_DATA (outbuf));
   }
@@ -684,7 +696,6 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
   GType type;
   AVInputFormat *in_plugin;
   GstFFMpegDemuxClassParams *params;
-  AVCodec *in_codec;
   gchar **extensions;
   GTypeInfo typeinfo = {
     sizeof (GstFFMpegDemuxClass),
@@ -718,7 +729,7 @@ gst_ffmpegdemux_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "mjpeg"))
       goto next;
 
-    if (!strcmp (in_plugin->name, "mov,mp4,m4a,3gp") ||
+    if (!strcmp (in_plugin->name, "mov,mp4,m4a,3gp,3g2") ||
         !strcmp (in_plugin->name, "avi") ||
         !strcmp (in_plugin->name, "asf") ||
         !strcmp (in_plugin->name, "mpegvideo") ||
