@@ -21,10 +21,8 @@
 
 #ifdef HAVE_FFMPEG_UNINSTALLED
 #include <avcodec.h>
-#include <avformat.h>
 #else
 #include <ffmpeg/avcodec.h>
-#include <ffmpeg/avformat.h>
 #endif
 
 #include <gst/gst.h>
