@@ -26,9 +26,9 @@
 #include "dsputil.h"
 
 #ifdef TEST
-#define av_log(p,l,...) fprintf(stderr,__VA_ARGS__)
 #undef printf
 #undef fprintf
+#define av_log(p,l,...) fprintf(stderr,__VA_ARGS__)
 #endif
 
 #define NB_COMPONENTS 3
@@ -560,6 +560,9 @@ ImgReSampleContext *img_resample_full_init(int owidth, int oheight,
 {
     ImgReSampleContext *s;
 
+    if (!owidth || !oheight || !iwidth || !iheight)
+	return NULL;
+
     s = av_mallocz(sizeof(ImgReSampleContext));
     if (!s)
         return NULL;
@@ -645,11 +648,15 @@ uint8_t img2[XSIZE1 * YSIZE1];
 
 void save_pgm(const char *filename, uint8_t *img, int xsize, int ysize)
 {
+#undef fprintf
     FILE *f;
     f=fopen(filename,"w");
     fprintf(f,"P5\n%d %d\n%d\n", xsize, ysize, 255);
     fwrite(img,1, xsize * ysize,f);
     fclose(f);
+#ifndef TEST
+#define fprintf please_use_av_log
+#endif
 }
 
 static void dump_filter(int16_t *filter)
